﻿using System;
using System.Runtime.Remoting.Contexts;
using System.Runtime.Remoting.Messaging;

namespace Earthworm
{
    /// <summary>
    /// 面向切面属性
    /// 此属性只可以内部使用
    /// 主要职责在于分配侦听接收器
    /// 以及实例化切面处理对象
    /// </summary>
    internal sealed class AspectOrientedProperty : IContextProperty, IDisposable, IContributeServerContextSink
    {
        /// <summary>
        /// 通知提供者对象所在的程序集
        /// </summary>
        private string assemblyName;
        /// <summary>
        /// 通知提供者的对象类型全名
        /// </summary>
        private string className;
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="assemblyName">程序集</param>
        /// <param name="className">对象类型全名</param>
        internal AspectOrientedProperty(string assemblyName, string className)
        {
            this.assemblyName = assemblyName;
            this.className = className;
        }
        /// <summary>
        /// 析构函数哈，省点
        /// </summary>
        ~AspectOrientedProperty()
        {
            Dispose();
        }
        /// <summary>
        /// 不解释
        /// </summary>
        public void Dispose()
        {
            assemblyName = null;
            className = null;
            GC.SuppressFinalize(this);
        }
        /// <summary>
        /// 当上下文冻结时调用
        /// 不需要，不做实现
        /// </summary>
        /// <param name="newContext">要冻结的上下文</param>
        public void Freeze(Context newContext){}
        /// <summary>
        ///  返回一个指示上下文属性是否与新上下文兼容的布尔值。
        /// </summary>
        /// <param name="newCtx">新上下文</param>
        /// <returns>如果该上下文属性可以与给定的上下文中的其他上下文属性共存，则为 true；否则为 false。此处给true</returns>
        public bool IsNewContextOK(Context newCtx)
        {
            return true;
        }
        /// <summary>
        /// 获取将属性添加到上下文中时使用的属性名称
        /// 就叫Earthworm吧
        /// </summary>
        public string Name
        {
            get { return "Earthworm"; }
        }
        /// <summary>
        /// 将第一个接收器放入到目前为止组成的接收器链中，然后将其消息接收器连接到已经形成的链前面
        /// </summary>
        /// <param name="nextSink">到目前为止组成的接收链</param>
        /// <returns>复合接收器链</returns>
        public IMessageSink GetServerContextSink(IMessageSink nextSink)
        {
            return new Aspect(nextSink,assemblyName,className);
        }
    }
}
